﻿using System;
using UnityEngine;
using UnityEditor;
using System.Collections;

[InitializeOnLoad]
static class EditorScript
{
    const string EnterPrefsKey = "__EnterTime0";
    const string LeavePrefsKey = "__LeaveTime0";


    static EditorScript()
    {
        EditorApplication.playmodeStateChanged += OnPlaymodeStateChanged;
    }

    [MenuItem("Test/Measure Open Current Scene", priority = 1)]
    static void MeasureOpenScene()
    {
        long ticks = DateTime.Now.Ticks;

        var scenePath = UnityEditor.SceneManagement.EditorSceneManager.GetActiveScene().path;
        UnityEditor.SceneManagement.EditorSceneManager.OpenScene(scenePath);

        var diff = TimeSpan.FromTicks(DateTime.Now.Ticks) - TimeSpan.FromTicks(ticks);
        Debug.Log(string.Format("Open scene took {0:F2} secs", diff.TotalSeconds));
    }

    [MenuItem("Test/Measure Open Current Scene", validate = true)]
    static bool MeasureOpenSceneValidate()
    {
        var scene = UnityEditor.SceneManagement.EditorSceneManager.GetActiveScene();
        return !string.IsNullOrEmpty(AssetDatabase.AssetPathToGUID(scene.path));
    }

    [MenuItem("Test/Measure Save Current Scene", priority = 2)]
    static void MeasureSaveScene()
    {
        long ticks = DateTime.Now.Ticks;

        UnityEditor.SceneManagement.EditorSceneManager.MarkAllScenesDirty();
        UnityEditor.SceneManagement.EditorSceneManager.SaveOpenScenes();

        var diff = TimeSpan.FromTicks(DateTime.Now.Ticks) - TimeSpan.FromTicks(ticks);
        Debug.Log(string.Format("Save scene took {0:F2} secs", diff.TotalSeconds));
    }

    [MenuItem("Test/Measure Save Current Scene", validate = true)]
    static bool MeasureSaveSceneValidate()
    {
        var scene = UnityEditor.SceneManagement.EditorSceneManager.GetActiveScene();
        return !string.IsNullOrEmpty(AssetDatabase.AssetPathToGUID(scene.path));
    }

    [MenuItem("Test/10000 x new GameObject()", priority = 100)]
    static void Add10000GameObjects()
    {
        long ticks = DateTime.Now.Ticks;

        for(var n = 0; n < 10000; ++n)
        {
            new GameObject();
        }

        var diff = TimeSpan.FromTicks(DateTime.Now.Ticks) - TimeSpan.FromTicks(ticks);
        Debug.Log(string.Format("10000 x new GameObject() took {0:F2} secs", diff.TotalSeconds));
    }

    [MenuItem("Test/10000 x InstantiatePrefab(Transform.prefab)", priority = 101)]
    static void Add10000Prefabs()
    {
        var prefab = AssetDatabase.LoadAssetAtPath<GameObject>("Assets/Transform.prefab");

        long ticks = DateTime.Now.Ticks;

        for(var n = 0; n < 10000; ++n)
        {
            var obj = (GameObject)PrefabUtility.InstantiatePrefab(prefab);
            obj.name = "New Game Object";
        }

        var diff = TimeSpan.FromTicks(DateTime.Now.Ticks) - TimeSpan.FromTicks(ticks);
        Debug.Log(string.Format("10000 x InstantiatePrefab(Transform.prefab) took {0:F2} secs", diff.TotalSeconds));
    }

    [MenuItem("Test/Count Root GameObjects in Scene", priority = 200)]
    static void CountGameObjectsInScene()
    {
        var scene = UnityEditor.SceneManagement.EditorSceneManager.GetActiveScene();
        Debug.LogFormat("{0} Root GameObjects in scene", scene.GetRootGameObjects().Length);
    }

    static void OnPlaymodeStateChanged()
    {
        // Measure enter playmode
        if(!EditorApplication.isPlaying && EditorApplication.isPlayingOrWillChangePlaymode)
        {
            long ticks = DateTime.Now.Ticks;
            EditorPrefs.SetString(EnterPrefsKey, ticks.ToString());
        }

        if(EditorApplication.isPlaying && EditorApplication.isPlayingOrWillChangePlaymode)
        {
            var ticksString = EditorPrefs.GetString(EnterPrefsKey);
            var ticks = long.Parse(ticksString);

            var diff = TimeSpan.FromTicks(DateTime.Now.Ticks) - TimeSpan.FromTicks(ticks);
            Debug.Log(string.Format("Enter Playmode took {0:F2} secs", diff.TotalSeconds));
        }
        

        // Measure leave playmode
        if(EditorApplication.isPlaying && !EditorApplication.isPlayingOrWillChangePlaymode)
        {
            long ticks = DateTime.Now.Ticks;
            EditorPrefs.SetString(LeavePrefsKey, ticks.ToString());
        }

        if(!EditorApplication.isPlaying && !EditorApplication.isPlayingOrWillChangePlaymode)
        {
            var ticksString = EditorPrefs.GetString(LeavePrefsKey);
            var ticks = long.Parse(ticksString);

            var diff = TimeSpan.FromTicks(DateTime.Now.Ticks) - TimeSpan.FromTicks(ticks);
            Debug.Log(string.Format("Leave Playmode took {0:F2} secs", diff.TotalSeconds));
        }
    }
}
