﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

public class Menu : MonoBehaviour
{
    public AudioListener MenuAudioListener;

    bool _Busy;
    TimeSpan _LoadPeriod;
    TimeSpan _UnloadPeriod;
    Vector2 _ScrollPosition;
    bool _UseAsync;

    void Awake()
    {
        // Keep this menu alive across scene changes
        DontDestroyOnLoad(gameObject);
    }

    void OnGUI()
    {
        GUI.enabled = !_Busy;
        GUILayout.Space(8);

        // Draw toggle to turn on/off async loading
        _UseAsync = GUILayout.Toggle(_UseAsync, "Load async");
        GUILayout.Space(8);

        // Draw the scene selection list
        using(var scrollView = new GUILayout.ScrollViewScope(_ScrollPosition, GUILayout.Width(300)))
        {
            _ScrollPosition = scrollView.scrollPosition;
            
            for(var n = 0; n < SceneManager.sceneCountInBuildSettings; ++n)
            {
                var scenePath = SceneUtility.GetScenePathByBuildIndex(n);
                var sceneName = System.IO.Path.GetFileNameWithoutExtension(scenePath);
                if(sceneName == "Menu" || sceneName == "Empty")
                    continue;

                if(GUILayout.Button(sceneName))
                {
                    if (_UseAsync)
                        this.StartCoroutine(LoadSceneAsync(sceneName));
                    else
                        this.StartCoroutine(LoadScene(sceneName));
                }
            }
        }
        GUILayout.Space(8);

        // Draw status
        GUI.enabled = true;
        GUILayout.BeginVertical(GUI.skin.window);
        if(_Busy)
        {
            GUILayout.Label("Busy...");
        }
        else
        {
            GUILayout.Label(string.Format("Unload scene took {0:F2} seconds.", _UnloadPeriod.TotalSeconds));
            GUILayout.Label(string.Format("Load scene took {0:F2} seconds.", _LoadPeriod.TotalSeconds));
        }
        GUILayout.EndVertical();

        // Add some animation, so we can see when it stutters
        var pivot = new Vector2(Screen.width - 200, Screen.height - 200);
        GUIUtility.RotateAroundPivot(100 * Time.unscaledTime, pivot);
        GUI.Box(new Rect(pivot.x - 128, pivot.y - 128, 256, 256), "ADVANCED\nSTUTTER\nDETECTION");
    }

    System.Collections.IEnumerator LoadScene(string sceneName)
    {
        MenuAudioListener.enabled = false;
        _Busy = true;

        // Unload blocking
        {
            long ticks = DateTime.Now.Ticks;
            SceneManager.LoadScene("Empty");
            yield return null;

            _UnloadPeriod = TimeSpan.FromTicks(DateTime.Now.Ticks) - TimeSpan.FromTicks(ticks);
        }

        // Load
        {
            long ticks = DateTime.Now.Ticks;
            SceneManager.LoadScene(sceneName);
            yield return null;

            _LoadPeriod = TimeSpan.FromTicks(DateTime.Now.Ticks) - TimeSpan.FromTicks(ticks);
        }

        _Busy = false;
    }

    System.Collections.IEnumerator LoadSceneAsync(string sceneName)
    {
        // Disable menu audio listener when loading a
        // scene to avoid 'multiple audiolisteners found' warning.
        MenuAudioListener.enabled = false;

        _Busy = true;

        // Unload blocking
        {
            long ticks = DateTime.Now.Ticks;
            yield return SceneManager.LoadSceneAsync("Empty");

            // The following call outputs a warning saying:
            // Unloading the last loaded scene <name>, is not supported
            //yield return SceneManager.UnloadSceneAsync(_LoadedScene);

            _UnloadPeriod = TimeSpan.FromTicks(DateTime.Now.Ticks) - TimeSpan.FromTicks(ticks);
        }

        // Load
        {
            long ticks = DateTime.Now.Ticks;
            yield return SceneManager.LoadSceneAsync(sceneName);

            _LoadPeriod = TimeSpan.FromTicks(DateTime.Now.Ticks) - TimeSpan.FromTicks(ticks);
        }

        _Busy = false;
    }
}
